<?php
namespace DDTrace\Contracts {
use DDTrace\Exceptions\InvalidReferencesSet;
use DDTrace\Exceptions\InvalidSpanOption;
use DDTrace\Exceptions\UnsupportedFormat;
use DDTrace\StartSpanOptions;
interface Tracer
{
    public function limited();
    public function getScopeManager();
    public function getActiveSpan();
    public function startActiveSpan($operationName, $options = array());
    public function startSpan($operationName, $options = array());
    public function inject(SpanContext $spanContext, $format, &$carrier);
    public function extract($format, $carrier);
    public function flush();
    public function setPrioritySampling($prioritySampling);
    public function getPrioritySampling();
    public function startRootSpan($operationName, $options = array());
    public function getRootScope();
    public function getSafeRootSpan();
    public function getTracesAsArray();
    public function getTracesCount();
}
}

namespace DDTrace\Contracts {
interface Span
{
    public function getOperationName();
    public function getContext();
    public function finish($finishTime = null);
    public function overwriteOperationName($newOperationName);
    public function setResource($resource);
    public function setTag($key, $value, $setIfFinished = false);
    public function getTag($key);
    public function log(array $fields = array(), $timestamp = null);
    public function addBaggageItem($key, $value);
    public function getBaggageItem($key);
    public function getAllBaggageItems();
    public function setError($error);
    public function setRawError($message, $type);
    public function hasError();
    public function getStartTime();
    public function getDuration();
    public function getTraceId();
    public function getSpanId();
    public function getParentId();
    public function getResource();
    public function getService();
    public function getType();
    public function isFinished();
    public function getAllTags();
    public function hasTag($name);
    public function setMetric($key, $value);
    public function getMetrics();
    public function setTraceAnalyticsCandidate($value = true);
    public function isTraceAnalyticsCandidate();
}
}

namespace DDTrace\Contracts {
interface Scope
{
    public function close();
    public function getSpan();
}
}

namespace DDTrace\Contracts {
interface ScopeManager
{
    const DEFAULT_FINISH_SPAN_ON_CLOSE = true;
    public function activate(Span $span, $finishSpanOnClose = self::DEFAULT_FINISH_SPAN_ON_CLOSE);
    public function getActive();
    public function close();
}
}

namespace DDTrace\Contracts {
use IteratorAggregate;
interface SpanContext extends IteratorAggregate
{
    public function getBaggageItem($key);
    public function withBaggageItem($key, $value);
    public function getAllBaggageItems();
    public function getPropagatedPrioritySampling();
    public function setPropagatedPrioritySampling($propagatedPrioritySampling);
    public function isHostRoot();
    public function getTraceId();
    public function getSpanId();
    public function getParentId();
    public function isDistributedTracingActivationContext();
}
}

namespace DDTrace\Log {
trait LoggingTrait
{
    protected static function logDebug($message, array $context = array())
    {
        Logger::get()->debug($message, $context);
    }
    protected static function logWarning($message, array $context = array())
    {
        Logger::get()->warning($message, $context);
    }
    protected static function logError($message, array $context = array())
    {
        Logger::get()->error($message, $context);
    }
    protected static function isLogDebugActive()
    {
        return Logger::get()->isLevelActive(LogLevel::DEBUG);
    }
}
}

namespace DDTrace\Data {
use DDTrace\Contracts\SpanContext as SpanContextInterface;
abstract class SpanContext implements SpanContextInterface
{
    public $traceId;
    public $spanId;
    public $parentId;
    public $isDistributedTracingActivationContext;
    public $propagatedPrioritySampling;
    public $origin;
    public $parentContext;
    public $baggageItems;
}
}

namespace DDTrace\Data {
use DDTrace\Time;
use DDTrace\Data\SpanContext as SpanContextData;
use DDTrace\Contracts\Span as SpanInterface;
if (PHP_VERSION_ID < 70000) {
    abstract class Span implements SpanInterface
    {
        public $operationName;
        public $context;
        public $resource;
        public $service;
        public $type;
        public $startTime;
        public $duration;
        public $tags = array();
        public $hasError = false;
        public $metrics = array();
        public $isTraceAnalyticsCandidate = false;
    }
} else {
    abstract class Span implements SpanInterface
    {
        public $context;
        public $hasError = false;
        protected $internalSpan;
        public function &__get($name)
        {
            if ($name == 'operationName') {
                $name = 'name';
            } elseif ($name == 'tags') {
                $name = 'meta';
            } elseif ($name == 'duration') {
                $duration = $this->internalSpan->getDuration();
                return $duration;
            } elseif ($name == 'startTime') {
                $startTime = $this->internalSpan->getStartTime();
                return $startTime;
            }
            return $this->internalSpan->{$name};
        }
        public function __set($name, $value)
        {
            if ($name == 'operationName') {
                $name = 'name';
            } elseif ($name == 'tags') {
                $name = 'meta';
            }
            return $this->internalSpan->{$name} = $value;
        }
        public function __isset($name)
        {
            return $this->__get($name) !== null;
        }
    }
}
}

namespace DDTrace {
use DDTrace\Contracts\Tracer as TracerInterface;
interface Transport
{
    public function send(TracerInterface $tracer);
    public function setHeader($key, $value);
}
}

namespace DDTrace\Obfuscation {
final class WildcardToRegex
{
    const REPLACEMENT_CHARACTER = '?';
    public static function convert($wildcardPattern)
    {
        $wildcardPattern = $replacement = trim($wildcardPattern);
        if (false !== strpos($replacement, '$*')) {
            $replacement = str_replace('%', '%%', $replacement);
            $replacementCount = 0;
            $replacement = str_replace('$*', '%s', $replacement, $replacementCount);
            $sprintfArgs = array($replacement);
            for ($i = 1; $i <= $replacementCount; $i++) {
                $sprintfArgs[] = '${' . $i . '}';
            }
            $replacement = call_user_func_array('sprintf', $sprintfArgs);
        }
        $replacement = str_replace('*', self::REPLACEMENT_CHARACTER, $replacement);
        return array('|^' . str_replace(array('\\$\\*', '\\*'), array('(.+)', '.+'), preg_quote($wildcardPattern, '|')) . '$|', $replacement);
    }
}
}

namespace DDTrace {
class Type
{
    const CACHE = 'cache';
    const HTTP_CLIENT = 'http';
    const WEB_SERVLET = 'web';
    const CLI = 'cli';
    const SQL = 'sql';
    const MESSAGE_CONSUMER = 'queue';
    const MESSAGE_PRODUCER = 'queue';
    const CASSANDRA = 'cassandra';
    const ELASTICSEARCH = 'elasticsearch';
    const MEMCACHED = 'memcached';
    const MONGO = 'mongodb';
    const REDIS = 'redis';
}
}

namespace DDTrace {
class Tag
{
    const ENV = 'env';
    const SPAN_TYPE = 'span.type';
    const SERVICE_NAME = 'service.name';
    const MANUAL_KEEP = 'manual.keep';
    const MANUAL_DROP = 'manual.drop';
    const PID = 'system.pid';
    const RESOURCE_NAME = 'resource.name';
    const DB_STATEMENT = 'sql.query';
    const ERROR = 'error';
    const ERROR_MSG = 'error.msg';
    const ERROR_TYPE = 'error.type';
    const ERROR_STACK = 'error.stack';
    const HTTP_METHOD = 'http.method';
    const HTTP_STATUS_CODE = 'http.status_code';
    const HTTP_URL = 'http.url';
    const LOG_EVENT = 'event';
    const LOG_ERROR = 'error';
    const LOG_ERROR_OBJECT = 'error.object';
    const LOG_MESSAGE = 'message';
    const LOG_STACK = 'stack';
    const TARGET_HOST = 'out.host';
    const TARGET_PORT = 'out.port';
    const BYTES_OUT = 'net.out.bytes';
    const ANALYTICS_KEY = '_dd1.sr.eausr';
    const HOSTNAME = '_dd.hostname';
    const ORIGIN = '_dd.origin';
    const VERSION = 'version';
    const SERVICE_VERSION = 'service.version';
    const ELASTICSEARCH_BODY = 'elasticsearch.body';
    const ELASTICSEARCH_METHOD = 'elasticsearch.method';
    const ELASTICSEARCH_PARAMS = 'elasticsearch.params';
    const ELASTICSEARCH_URL = 'elasticsearch.url';
    const MONGODB_BSON_ID = 'mongodb.bson.id';
    const MONGODB_COLLECTION = 'mongodb.collection';
    const MONGODB_DATABASE = 'mongodb.db';
    const MONGODB_PROFILING_LEVEL = 'mongodb.profiling_level';
    const MONGODB_READ_PREFERENCE = 'mongodb.read_preference';
    const MONGODB_SERVER = 'mongodb.server';
    const MONGODB_TIMEOUT = 'mongodb.timeout';
    const MONGODB_QUERY = 'mongodb.query';
    const REDIS_RAW_COMMAND = 'redis.raw_command';
}
}

namespace DDTrace {
use DDTrace\Contracts\Span as SpanInterface;
use DDTrace\Contracts\SpanContext as SpanContextInterface;
use DDTrace\Exceptions\InvalidReferenceArgument;
final class Reference
{
    const CHILD_OF = 'child_of';
    const FOLLOWS_FROM = 'follows_from';
    private $type;
    private $context;
    private function __construct($type, SpanContextInterface $context)
    {
        $this->type = $type;
        $this->context = $context;
    }
    public static function create($type, $context)
    {
        if (empty($type)) {
            throw InvalidReferenceArgument::forEmptyType();
        }
        return new self($type, self::extractContext($context));
    }
    public function getContext()
    {
        return $this->context;
    }
    public function isType($type)
    {
        return $this->type === $type;
    }
    private static function extractContext($context)
    {
        if ($context instanceof SpanContextInterface) {
            return $context;
        }
        if ($context instanceof SpanInterface) {
            return $context->getContext();
        }
        throw InvalidReferenceArgument::forInvalidContext($context);
    }
}
}

namespace DDTrace\Exceptions {
use InvalidArgumentException;
final class InvalidReferenceArgument extends InvalidArgumentException
{
    public static function forEmptyType()
    {
        return new self('Reference type can not be an empty string');
    }
    public static function forInvalidContext($context)
    {
        return new self(sprintf('Reference expects \\DDTrace\\Contracts\\Span or \\DDTrace\\Contracts\\SpanContext as context, got %s', is_object($context) ? get_class($context) : gettype($context)));
    }
}
}

namespace DDTrace\Exceptions {
use UnexpectedValueException;
final class UnsupportedFormat extends UnexpectedValueException
{
    public static function forFormat($format)
    {
        return new self(sprintf('The format \'%s\' is not supported.', $format));
    }
}
}

namespace DDTrace\Exceptions {
use InvalidArgumentException;
final class InvalidSpanArgument extends InvalidArgumentException
{
    public static function forTagKey($key)
    {
        return new self(sprintf('Invalid key type in given span tags. Expected string, got %s.', gettype($key)));
    }
    public static function forError($error)
    {
        return new self(sprintf('Error should be either Exception or Throwable, got %s.', gettype($error)));
    }
}
}

namespace DDTrace\Exceptions {
use DomainException;
final class InvalidReferencesSet extends DomainException
{
    public static function create($message)
    {
        return new self($message);
    }
    public static function forMoreThanOneParent()
    {
        return new self('Span can not have more than one parent, either one as child_of or either one as follows_from');
    }
}
}

namespace DDTrace\Exceptions {
use InvalidArgumentException;
final class InvalidSpanOption extends InvalidArgumentException
{
    public static function forIncludingBothChildOfAndReferences()
    {
        return new self('Either "childOf" or "references" options are accepted but not both.');
    }
    public static function forInvalidReference($reference)
    {
        return new self(sprintf('Invalid reference. Expected \\DDTrace\\Reference, got %s.', is_object($reference) ? get_class($reference) : gettype($reference)));
    }
    public static function forInvalidStartTime()
    {
        return new self(sprintf('Invalid start_time option. Expected int or float got string.'));
    }
    public static function forInvalidChildOf($childOfOption)
    {
        return new self(sprintf('Invalid child_of option. Expected Span or SpanContext, got %s', is_object($childOfOption) ? get_class($childOfOption) : gettype($childOfOption)));
    }
    public static function forUnknownOption($key)
    {
        return new self(sprintf('Invalid option %s.', $key));
    }
    public static function forInvalidTag($tag)
    {
        return new self(sprintf('Invalid tag. Expected string, got %s', gettype($tag)));
    }
    public static function forInvalidTagValue($tagValue)
    {
        return new self(sprintf('Invalid tag value. Expected scalar or object with __toString method, got %s', is_object($tagValue) ? get_class($tagValue) : gettype($tagValue)));
    }
    public static function forInvalidTags($value)
    {
        return new self(sprintf('Invalid tags value. Expected a associative array of tags, got %s', is_object($value) ? get_class($value) : gettype($value)));
    }
    public static function forInvalidReferenceSet($value)
    {
        return new self(sprintf('Invalid references set. Expected Reference or Reference[], got %s', is_object($value) ? get_class($value) : gettype($value)));
    }
    public static function forFinishSpanOnClose($value)
    {
        return new self(sprintf('Invalid type for finish_span_on_close. Expected bool, got %s', is_object($value) ? get_class($value) : gettype($value)));
    }
    public static function forIgnoreActiveSpan($value)
    {
        return new self(sprintf('Invalid type for ignore_active_span. Expected bool, got %s', is_object($value) ? get_class($value) : gettype($value)));
    }
}
}

namespace DDTrace {
use DDTrace\Contracts\Tracer as TracerInterface;
final class GlobalTracer
{
    private static $instance;
    public static function set(TracerInterface $tracer)
    {
        self::$instance = $tracer;
    }
    public static function get()
    {
        if (null !== self::$instance) {
            return self::$instance;
        }
        return self::$instance = NoopTracer::create();
    }
}
}

namespace DDTrace\Http {
use DDTrace\Obfuscation\WildcardToRegex;
class Urls
{
    private static $defaultPatterns = array('<(/)([0-9a-f]{8}-?[0-9a-f]{4}-?[1-5][0-9a-f]{3}-?[89ab][0-9a-f]{3}-?[0-9a-f]{12})(/|$)>i', '<(/)([0-9a-f]{8,128})(/|$)>i', '<(/)([0-9]+)(/|$)>');
    private $replacementPatterns = array();
    public function __construct(array $patternsWithWildcards = array())
    {
        foreach ($patternsWithWildcards as $pattern) {
            $this->replacementPatterns[] = WildcardToRegex::convert($pattern);
        }
    }
    public static function sanitize($url)
    {
        return strstr($url, '?', true) ?: $url;
    }
    public static function hostname($url)
    {
        return (string) parse_url($url, PHP_URL_HOST);
    }
    public static function hostnameForTag($url)
    {
        return 'host-' . self::hostname($url);
    }
    public function normalize($url)
    {
        $url = self::sanitize($url);
        foreach ($this->replacementPatterns as $regexReplacement) {
            list($regex, $replacement) = $regexReplacement;
            $replacedCount = 0;
            $url = preg_replace($regex, $replacement, $url, -1, $replacedCount);
            if ($replacedCount > 0) {
                return $url;
            }
        }
        return preg_replace(self::$defaultPatterns, '$1?$3', $url);
    }
}
}

namespace DDTrace\Log {
interface LoggerInterface
{
    public function debug($message, array $context = array());
    public function warning($message, array $context = array());
    public function error($message, array $context = array());
    public function isLevelActive($level);
}
}

namespace DDTrace\Log {
trait InterpolateTrait
{
    public function interpolate($message, array $context = array())
    {
        $replace = array();
        foreach ($context as $key => $val) {
            if (!is_array($val) && (!is_object($val) || method_exists($val, '__toString'))) {
                $replace['{' . $key . '}'] = $val;
            }
        }
        return strtr($message, $replace);
    }
}
}

namespace DDTrace\Log {
final class LogLevel
{
    const EMERGENCY = 'emergency';
    const ALERT = 'alert';
    const CRITICAL = 'critical';
    const ERROR = 'error';
    const WARNING = 'warning';
    const NOTICE = 'notice';
    const INFO = 'info';
    const DEBUG = 'debug';
    public static function all()
    {
        return array(self::EMERGENCY, self::ALERT, self::CRITICAL, self::ERROR, self::WARNING, self::NOTICE, self::INFO, self::DEBUG);
    }
}
}

namespace DDTrace\Log {
abstract class AbstractLogger implements LoggerInterface
{
    private $enabledLevels = array();
    public function __construct($level)
    {
        $level = trim(strtolower($level));
        $enabled = false;
        foreach (array_reverse(LogLevel::all()) as $knownLevel) {
            $enabled = $enabled || $level === $knownLevel;
            $this->enabledLevels[$knownLevel] = $enabled;
        }
    }
    public function isLevelActive($level)
    {
        return (bool) $this->enabledLevels[$level];
    }
}
}

namespace DDTrace\Log {
class ErrorLogLogger extends AbstractLogger
{
    use InterpolateTrait;
    public function debug($message, array $context = array())
    {
        $this->emit(LogLevel::DEBUG, $message, $context);
    }
    public function warning($message, array $context = array())
    {
        $this->emit(LogLevel::WARNING, $message, $context);
    }
    public function error($message, array $context = array())
    {
        $this->emit(LogLevel::ERROR, $message, $context);
    }
    private function emit($level, $message, array $context = array())
    {
        if (!$this->isLevelActive($level)) {
            return;
        }
        $interpolatedMessage = $this->interpolate($message, $context);
        $date = date(\DateTime::ATOM);
        error_log("[{$date}] [ddtrace] [{$level}] - {$interpolatedMessage}");
    }
}
}

namespace DDTrace\Log {
final class NullLogger extends AbstractLogger
{
    public function debug($message, array $context = array())
    {
    }
    public function warning($message, array $context = array())
    {
    }
    public function error($message, array $context = array())
    {
    }
    public function isLevelActive($level)
    {
        return false;
    }
}
}

namespace DDTrace {
class Format
{
    const BINARY = 'binary';
    const TEXT_MAP = 'text_map';
    const HTTP_HEADERS = 'http_headers';
    const CURL_HTTP_HEADERS = 'curl_http_headers';
}
}

namespace DDTrace {
use DDTrace\Contracts\ScopeManager as ScopeManagerInterface;
use DDTrace\Contracts\Span as SpanInterface;
use DDTrace\Contracts\SpanContext as SpanContextInterface;
use DDTrace\Exceptions\InvalidReferencesSet;
use DDTrace\Exceptions\InvalidSpanOption;
final class StartSpanOptions
{
    private $references = array();
    private $tags = array();
    private $startTime;
    private $finishSpanOnClose = ScopeManagerInterface::DEFAULT_FINISH_SPAN_ON_CLOSE;
    private $ignoreActiveSpan = false;
    public static function create(array $options)
    {
        $spanOptions = new self();
        foreach ($options as $key => $value) {
            switch ($key) {
                case 'child_of':
                    if (!empty($spanOptions->references)) {
                        throw InvalidSpanOption::forIncludingBothChildOfAndReferences();
                    }
                    $spanOptions->references[] = self::buildChildOf($value);
                    break;
                case 'references':
                    if (!empty($spanOptions->references)) {
                        throw InvalidSpanOption::forIncludingBothChildOfAndReferences();
                    }
                    if ($value instanceof Reference) {
                        $spanOptions->references = array($value);
                    } elseif (is_array($value)) {
                        $spanOptions->references = self::buildReferences($value);
                    } else {
                        throw InvalidSpanOption::forInvalidReferenceSet($value);
                    }
                    break;
                case 'tags':
                    if (!is_array($value)) {
                        throw InvalidSpanOption::forInvalidTags($value);
                    }
                    foreach ($value as $tag => $tagValue) {
                        if ($tag !== (string) $tag) {
                            throw InvalidSpanOption::forInvalidTag($tag);
                        }
                        $spanOptions->tags[$tag] = $tagValue;
                    }
                    break;
                case 'start_time':
                    if (is_scalar($value) && !is_numeric($value)) {
                        throw InvalidSpanOption::forInvalidStartTime();
                    }
                    $spanOptions->startTime = $value;
                    break;
                case 'finish_span_on_close':
                    if (!is_bool($value)) {
                        throw InvalidSpanOption::forFinishSpanOnClose($value);
                    }
                    $spanOptions->finishSpanOnClose = $value;
                    break;
                case 'ignore_active_span':
                    if (!is_bool($value)) {
                        throw InvalidSpanOption::forIgnoreActiveSpan($value);
                    }
                    $spanOptions->ignoreActiveSpan = $value;
                    break;
                default:
                    throw InvalidSpanOption::forUnknownOption($key);
                    break;
            }
        }
        return $spanOptions;
    }
    public function withParent($parent)
    {
        $newSpanOptions = new StartSpanOptions();
        $newSpanOptions->references[] = self::buildChildOf($parent);
        $newSpanOptions->tags = $this->tags;
        $newSpanOptions->startTime = $this->startTime;
        $newSpanOptions->finishSpanOnClose = $this->finishSpanOnClose;
        $newSpanOptions->ignoreActiveSpan = $this->ignoreActiveSpan;
        return $newSpanOptions;
    }
    public function getReferences()
    {
        return $this->references;
    }
    public function getTags()
    {
        return $this->tags;
    }
    public function getStartTime()
    {
        return $this->startTime;
    }
    public function shouldFinishSpanOnClose()
    {
        return $this->finishSpanOnClose;
    }
    public function shouldIgnoreActiveSpan()
    {
        return $this->ignoreActiveSpan;
    }
    private static function buildChildOf($value)
    {
        if ($value instanceof SpanInterface) {
            return Reference::create(Reference::CHILD_OF, $value->getContext());
        }
        if ($value instanceof SpanContextInterface) {
            return Reference::create(Reference::CHILD_OF, $value);
        }
        if ($value instanceof \DDTrace\OpenTracer\SpanContext || $value instanceof \DDTrace\OpenTracer1\SpanContext) {
            return Reference::create(Reference::CHILD_OF, $value->unwrapped());
        }
        throw InvalidSpanOption::forInvalidChildOf($value);
    }
    private static function buildReferences(array $referencesArray)
    {
        $references = array();
        foreach ($referencesArray as $reference) {
            if (!$reference instanceof Reference) {
                throw InvalidSpanOption::forInvalidReference($reference);
            }
            $references[] = $reference;
        }
        return $references;
    }
}
}

